@@ -296,14 +296,16 @@ ZX_Z_normalize(GEN pol, GEN *ptk)
   long i,j, sk, n = degpol(pol); /* > 0 */
   GEN k, fa, P, E, a, POL;
 
+  if (ptk) *ptk = gen_1;
+  if (!n) return pol;
   a = pol + 2; k = gel(a,n-1); /* a[i] = coeff of degree i */
   for (i = n-2; i >= 0; i--)
   {
     k = gcdii(k, gel(a,i));
-    if (is_pm1(k)) { if (ptk) *ptk = gen_1; return pol; }
+    if (is_pm1(k)) return pol;
   }
   sk = signe(k);
-  if (!sk) { if (ptk) *ptk = gen_1; return pol; /* monomial! */ }
+  if (!sk) return pol; /* monomial! */
   fa = absZ_factor_limit(k, 0); k = gen_1;
   P = gel(fa,1);
   E = gel(fa,2);
@@ -329,7 +331,8 @@ ZX_Z_normalize(GEN pol, GEN *ptk)
       gel(a,j) = diviiexact(gel(a,j), pvj);
     }
   }
-  if (ptk) *ptk = k; return POL;
+  if (ptk) *ptk = k;
+  return POL;
 }
 
 /* Assume pol != 0 in Z[X]. Find C in Q, L in Z such that POL = C pol(x/L) monic
@@ -783,6 +786,7 @@ idealramgroupindex(GEN nf, GEN gal, GEN pr)
     iso = gel(grp,i); o = perm_order(iso);
     if (rorder%o == 0)
     {
+      pari_sp av = avma;
       GEN piso = iso;
       GEN S = poltobasis(nf, galoispermtopol(gal, iso));
       GEN pi = pr_get_gen(pr);
@@ -803,6 +807,7 @@ idealramgroupindex(GEN nf, GEN gal, GEN pr)
         piso = perm_mul(piso,iso);
         if(cgcd(j,o)==1) idx[piso[1]] = idx[i];
       }
+      avma = av;
     }
   }
   return gerepileuptoleaf(av, idx);
