@@ -2122,8 +2122,10 @@ sin_p(GEN x)
 static GEN
 cxexp(GEN x, long prec)
 {
-  GEN r,p1,p2, y = cgetg(3,t_COMPLEX);
+  GEN r, p1, p2, y = cgetg(3,t_COMPLEX);
   pari_sp av = avma, tetpil;
+  long l;
+  l = precision(x); if (l > prec) prec = l;
   r = gexp(gel(x,1),prec);
   if (gequal0(r)) { gel(y,1) = r; gel(y,2) = r; return y; }
   gsincos(gel(x,2),&p2,&p1,prec);
@@ -2794,6 +2796,7 @@ glog(GEN x, long prec)
     }
     case t_COMPLEX:
       if (ismpzero(gel(x,2))) return glog(gel(x,1), prec);
+      l = precision(x); if (l > prec) prec = l;
       if (ismpzero(gel(x,1)))
       {
         GEN a = gel(x,2), b;
@@ -2802,7 +2805,6 @@ glog(GEN x, long prec)
         a = isint1(a) ? gen_0: glog(a,prec);
         return gerepilecopy(av, mkcomplex(a, b));
       }
-      l = precision(x); if (l > prec) prec = l;
       if (prec >= LOGAGMCX_LIMIT) return logagmcx(x, prec);
       y = cgetg(3,t_COMPLEX);
       gel(y,2) = garg(x,prec);
@@ -2992,8 +2994,8 @@ gcos(GEN x, long prec)
     case t_REAL: return mpcos(x);
     case t_COMPLEX:
       if (isintzero(gel(x,1))) return gcosh(gel(x,2), prec);
-      i = precision(x); if (!i) i = prec;
-      y = cgetc(i); av = avma;
+      i = precision(x); if (i) prec = i;
+      y = cgetc(prec); av = avma;
       r = gexp(gel(x,2),prec);
       v1 = gmul2n(addrr(invr(r),r), -1); /* = cos(I*Im(x)) */
       u1 = subrr(v1, r); /* = - I*sin(I*Im(x)) */
@@ -3055,8 +3057,8 @@ gsin(GEN x, long prec)
     case t_REAL: return mpsin(x);
     case t_COMPLEX:
       if (isintzero(gel(x,1))) retmkcomplex(gen_0,gsinh(gel(x,2),prec));
-      i = precision(x); if (!i) i = prec;
-      y = cgetc(i); av = avma;
+      i = precision(x); if (i) prec = i;
+      y = cgetc(prec); av = avma;
       r = gexp(gel(x,2),prec);
       v1 = gmul2n(addrr(invr(r),r), -1); /* = cos(I*Im(x)) */
       u1 = subrr(r, v1); /* = I*sin(I*Im(x)) */
@@ -3176,6 +3178,8 @@ cxexpm1(GEN z, long prec)
 {
   pari_sp av = avma;
   GEN X, Y, x = real_i(z), y = imag_i(z);
+  long l = precision(z);
+  if (l) prec = l;
   if (typ(x) != t_REAL) x = gtofp(x, prec);
   if (typ(y) != t_REAL) y = gtofp(y, prec);
   if (gequal0(y)) return mpexpm1(x);
@@ -3207,9 +3211,9 @@ gsincos(GEN x, GEN *s, GEN *c, long prec)
       mpsincos(x,s,c); return;
 
     case t_COMPLEX:
-      i = precision(x); if (!i) i = prec;
-      ps = cgetc(i); *s = ps;
-      pc = cgetc(i); *c = pc; av = avma;
+      i = precision(x); if (i) prec = i;
+      ps = cgetc(prec); *s = ps;
+      pc = cgetc(prec); *c = pc; av = avma;
       r = gexp(gel(x,2),prec);
       v1 = gmul2n(addrr(invr(r),r), -1); /* = cos(I*Im(x)) */
       u1 = subrr(r, v1); /* = I*sin(I*Im(x)) */
@@ -3318,8 +3322,8 @@ gsinc(GEN x, long prec)
         av = avma;
         return gerepileuptoleaf(av,gdiv(gsinh(gel(x,2),prec),gel(x,2)));
       }
-      i = precision(x); if (!i) i = prec;
-      y = cgetc(i); av = avma;
+      i = precision(x); if (i) prec = i;
+      y = cgetc(prec); av = avma;
       r = gexp(gel(x,2),prec);
       v1 = gmul2n(addrr(invr(r),r), -1); /* = cos(I*Im(x)) */
       u1 = subrr(r, v1); /* = I*sin(I*Im(x)) */
